"use client";

import React, {useEffect, useState} from 'react';
import Web3 from 'web3';
import {
    Container,
    Typography,
    Card,
    CardContent,
    List,
    ListItem,
    Paper,
    Grid,
    Button,
    Divider,
    Box
} from '@mui/material';
import {useRouter} from 'next/navigation';
import Link from 'next/link';

import NavBar from '../components/NavBar';
import {checkConnectionMetaMask, getAuctionContract} from '../utils';

const AuctionList = () => {
    const [auctions, setAuctions] = useState([]);
    const {auctionContract, web3} = getAuctionContract();
    const [passedAuctions, setPassedAuctions] = useState([]);
    const router = useRouter();

    useEffect(() => {
        const fetchItemsAvailable = async () => {
            const accounts = await checkConnectionMetaMask();
            if (accounts.length === 0) {
                router.push('/');
            } else {
                try {
                    await auctionContract.methods.getAllAuctionItems().send({ from: window.ethereum.selectedAddress, gas: 3000000 });
                    const availableAuctions = await auctionContract.methods.getAllAuctionItems().call();
                    console.log('Available Auctions:', availableAuctions);

                    setAuctions(availableAuctions);
                } catch (error) {
                    console.error('Error fetching available auctions:', error);
                    setAuctions([]);
                }
                try {
                    const auctionsPassed = await auctionContract.methods.getAuctionPassed().call();
                    console.log('Auctions Passed:', auctionsPassed);

                    const passedAuctionsData = auctionsPassed.map(auction => ({
                        name: auction.name,
                        status: auction.buyer === '0x0000000000000000000000000000000000000000' ? 'Perdue' : 'Gagnée',
                    }));
                    setPassedAuctions(passedAuctionsData);
                } catch (error) {
                    console.error('Error fetching passed auctions:', error);
                    setPassedAuctions([]);
                }
            };
        }
        fetchItemsAvailable();
    }, []);

const formatTimestamp = (timestamp) => {
    const date = new Date(Number(timestamp) * 1000);
    const formattedDate = date.toLocaleString('fr-FR');
    return formattedDate;
};

const handleViewDetails = (auctionId) => {
    const id = passedAuctions.length + auctionId;
    router.push(`/auction-list/${id + 1}`);
};

const handleBid = async (auctionId) => {
    try {
        const id = passedAuctions.length + auctionId;


        await auctionContract.methods.buy().send({
            from: window.ethereum.selectedAddress,
            value: web3.utils.toWei(auctions[auctionId].price, 'ether'),
            gas: 30000,
        });

        console.log("Auction successful!");
    } catch (error) {
        console.log("Auction ID:", auctions[auctionId]);

        console.error("Error during auction:", error.message);
    }
};

return (
    <>
        <NavBar/>
        <Container>
            <Typography variant="h4">Liste des Enchères Disponibles</Typography>
            <List>
                {auctions.map((auction, index) => (
                    <React.Fragment key={index}>
                        <ListItem>
                            <Typography variant="subtitle1">
                                Nom: {auction.name}
                                <br/>
                                Prix: {web3.utils.fromWei(auction.price.toString(), 'ether')} Ether
                                <br/>
                                Timestamp: {formatTimestamp(auction.timestamp)}
                                <br/>
                                Statut: {auction.status}
                            </Typography>
                        </ListItem>
                        <ListItem>
                            <Button onClick={() => handleViewDetails(index)} variant="contained"
                                    color="primary">
                                Voir détail
                            </Button>
                            {auction.status === 'En Cours' && (
                                <>
                                    <Box m={1}/>
                                    <Button onClick={() => handleBid(index)} variant="contained"
                                            color="secondary">
                                        Enchérir avec le prix actuel
                                    </Button>
                                </>
                            )}
                        </ListItem>
                        <Divider/>
                    </React.Fragment>
                ))}
            </List>
            <Typography variant="h4">Liste des Enchères Passées</Typography>
            <List>
                {passedAuctions.map((auction, index) => (
                    <React.Fragment key={index}>
                        <ListItem>
                            <Typography variant="subtitle1">
                                Nom: {auction.name}
                                <br/>
                                Statut: {auction.status}
                            </Typography>
                        </ListItem>
                        <Divider/>
                    </React.Fragment>
                ))}
            </List>
        </Container>
    </>
);
}
;

export default AuctionList;
